var fs = require('fs'),
	path = require('path');

module.exports = {
	createTlsConfig: function (config) {
		if (!config.$(this.parameters.tlsEnabled)) {
			return {};
		}

		var tlsConfig = {
				keyPath: config.$(this.parameters.tlsKeyPath),
				certPath: config.$(this.parameters.tlsCertPath),
				pfxPath: config.$(this.parameters.tlsPfxPath),
				requestCert: config.$(this.parameters.tlsRequestCert),
				caPaths: config.$(this.parameters.tlsCaPaths)
			},
			absoluteKeyPath,
			absoluteCertPath,
			absolutePfxPath,
			absoluteCaPath,
			ca = [];

		if (tlsConfig.keyPath) {
			absoluteKeyPath = path.resolve(tlsConfig.keyPath);
			tlsConfig.key = fs.readFileSync(absoluteKeyPath);
		}

		if (tlsConfig.certPath) {
			absoluteCertPath = path.resolve(tlsConfig.certPath);
			tlsConfig.cert = fs.readFileSync(absoluteCertPath);
		}

		if (tlsConfig.pfxPath) {
			absolutePfxPath = path.resolve(tlsConfig.pfxPath);
			tlsConfig.pfx = fs.readFileSync(absolutePfxPath);
		}

		if (tlsConfig.caPaths) {

			tlsConfig.caPaths.forEach(function (item) {
				absoluteCaPath = path.resolve(item);
				ca.push(fs.readFileSync(absoluteCaPath));
			});
			tlsConfig.ca = ca;
		}

		return tlsConfig;
	},

	parameters: {
		charset: 'charset',
		serverAddress: 'eventBus.serverAddress',
		serverPort: 'eventBus.serverPort',
		reconnectInterval: 'eventBus.reconnectInterval',
		sendMessageQueueInterval: 'eventBus.sendMessageQueueInterval',
		sendMessagePerTimeLimit: 'eventBus.sendMessagePerTimeLimit',
		masterSecret: 'eventBus.masterSecret',
		workerSecret: 'eventBus.workerSecret',
		//tls subsection
		tlsEnabled: 'eventBus.tls.enabled',
		tlsKeyPath: 'eventBus.tls.keyPath',
		tlsCertPath: 'eventBus.tls.certPath',
		tlsPfxPath: 'eventBus.tls.pfxPath',
		tlsRequestCert: 'eventBus.tls.requestCert',
		tlsCaPaths: 'eventBus.tls.caPaths'
	},

	messageTypes: {
		data: 'data',
		sync: 'sync',
		shared: 'shared',
		load: 'load',
		address: 'address'
	}

};
