/* 
 * neutrino
 *
 * Copyright (c) 2012 Denis Rechkunov and project contributors.
 *
 * neutrino's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * This license applies to all parts of neutrino that are not externally
 * maintained libraries.
 */

module.exports = Balancer;

/**
 * Creates new instance of cluster balancer.
 * @constructor
 */
function Balancer() {

	this.workersByAddress_ = {};
}

/**
 * Hashtable of cluster worker nodes.
 * @type {Object}
 * @private
 */
Balancer.prototype.workersByAddress_ = null;

/**
 * Adds new worker node to balancer.
 * @param {String} workerId Worker node ID.
 */
Balancer.prototype.addWorker = function (workerId) {
	this.setWeight(workerId, 0);
};

/**
 * Removes worker node from balancer.
 * @param {String} workerId Worker node ID.
 */
Balancer.prototype.removeWorker = function (workerId) {
	delete this.workersByAddress_[workerId];
};

/**
 * Sets load weight of worker node.
 * @param {String} workerId Worker node ID.
 * @param {Number} weight Load weight.
 */
Balancer.prototype.setWeight = function (workerId, weight) {
	this.workersByAddress_[workerId] = weight;
};

/**
 * Gets most free worker node ID.
 * @return {String}
 */
Balancer.prototype.getFreeWorkerId = function () {
	var minKey,
		minValue = Number.MAX_VALUE;

	for (var key in this.workersByAddress_) {

		if (!this.workersByAddress_.hasOwnProperty(key)) {
			continue;
		}

		if (this.workersByAddress_[key] >= minValue) {
			continue;
		}
		minKey = key;
		minValue = this.workersByAddress_[key];
	}

	this.workersByAddress_[minKey]++;
	return minKey || null;
};
