/* 
 * neutrino
 *
 * Copyright (c) 2012 Denis Rechkunov and project contributors.
 *
 * neutrino's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * This license applies to all parts of neutrino that are not externally
 * maintained libraries.
 */

module.exports = LimitedQueue;

/**
 * Creates new instance of limited queue.
 * @param {Number} size Max size of queue.
 * @constructor
 */
function LimitedQueue(size) {
	this.size_ = size;
	this.queue_ = [];
}

/**
 * Max size of queue.
 * @type {Number}
 * @private
 */
LimitedQueue.prototype.size_ = 0;

/**
 * Current queue array.
 * @type {Array}
 * @private
 */
LimitedQueue.prototype.queue_ = null;

/**
 * Adds specified object to queue.
 * @param {Object} object Specified object to add.
 */
LimitedQueue.prototype.add = function (object) {

	if (this.queue_.length == this.size_) {
		this.queue_.shift();
	}

	this.queue_.push(object);
};

/**
 * Removes specified object from queue.
 * @param {Object} object Specified object to remove.
 * @return {Boolean}
 */
LimitedQueue.prototype.remove = function (object) {

	var index = this.queue_.indexOf(object);

	if (index === -1) {
		return false;
	}

	return this.queue_.splice(index, 1).length > 0;
};

/**
 * Removes element at specified index.
 * @param {Number} index Index of element to remove.
 * @return {Boolean}
 */
LimitedQueue.prototype.removeAt = function (index) {

	if (index < 0 || index >= this.queue_.length) {
		return false;
	}

	return this.queue_.splice(index, 1).length === 1;
};