/* 
 * neutrino
 *
 * Copyright (c) 2012 Denis Rechkunov and project contributors.
 *
 * neutrino's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * This license applies to all parts of neutrino that are not externally
 * maintained libraries.
 */

var MasterNode = require('./cluster/master/master-node.js'),
	WorkerNode = require('./cluster/worker/worker-node.js'),
	singleton = require('./common/patterns/singleton.js'),
	Config = require('./core/config.js'),
	Logger = require('./security/logger.js');

exports.version = '0.0.2';

/**
 * Configure neutrino for specified config object or file path.
 * @param {String|Object} config Config object of file path.
 */
exports.configure = function (config) {

	singleton.clear();
	exports.currentConfig = config ? new Config(config) : new Config();
	exports.logger = new Logger(exports.currentConfig);

};

/**
 * Create and start new neutrino master node instance.
 * @param {Object} config Configuration object (Optional).
 * @return {MasterNode}
 */
exports.createMaster = function (config) {

	var launchConfig = config ? new Config(config) : exports.currentConfig,
		master = new MasterNode(launchConfig);
	master.start();
	return master;

};

/**
 * Create and start new neutrino worker node instance.
 * @param {Object} config Configuration object (Optional).
 * @return {WorkerNode}
 */
exports.createWorker = function (config) {

	var launchConfig = config ? new Config(config) : exports.currentConfig,
		worker = new WorkerNode(launchConfig);
	worker.on(WorkerNode.events.error, function (error) {
		exports.logger.error(error);
	});
	worker.start();
	return worker;

};

process.on('uncaughtException', function (error) {
	try {
		exports.logger.error(error);
	} catch (e) {
		throw e;
	}
});