/*
 * neutrino
 *
 * Copyright (c) 2012 Denis Rechkunov and project contributors.
 *
 * neutrino's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * This license applies to all parts of neutrino that are not externally
 * maintained libraries.
 */

var neutrino = require('../../index.js'),
	LimitedQueue = require('../../common/collections/limited-queue.js');

exports['Queue add with limit'] = function (test) {

	test.expect(4);

	var queue = new LimitedQueue(3);

	for (var i = 0; i < 5; i++) {
		queue.add({index: i});
	}
	test.deepEqual(queue.queue_[0].index, 2);
	test.deepEqual(queue.queue_[1].index, 3);
	test.deepEqual(queue.queue_[2].index, 4);
	test.deepEqual(queue.queue_.length, 3);

	test.done();
};

exports['Remove object from queue'] = function (test) {

	test.expect(4);

	var queue = new LimitedQueue(3);

	for (var i = 0; i < 5; i++) {
		queue.add({index: i});
	}

	test.deepEqual(queue.queue_.length, 3);

	var obj = queue.queue_[2],
		isRemoved = queue.remove(obj),
		isDummyRemoved = queue.remove({test: 123});

	test.deepEqual(queue.queue_.length, 2);

	test.deepEqual(isRemoved, true);
	test.deepEqual(isDummyRemoved, false);
	test.done();
};

exports['Remove from queue by index'] = function (test) {

	test.expect(5);

	var queue = new LimitedQueue(3);

	for (var i = 0; i < 5; i++) {
		queue.add({index: i});
	}

	test.deepEqual(queue.queue_.length, 3);

	var isRemovedNegative = queue.removeAt(-1),
		isRemovedOverLength = queue.removeAt(3),
		isRemovedCorrect = queue.removeAt(2);

	test.deepEqual(isRemovedNegative, false);
	test.deepEqual(isRemovedOverLength, false);
	test.deepEqual(isRemovedCorrect, true);
	test.deepEqual(queue.queue_.length, 2);
	test.done();

};
